<?php

namespace App\Base\Security;


use Predis\Client;

class BruteforceStorage
{
    private $redis;
    private $prefixKnown = 'bruteforce|known|ip|';
    private $prefixKnownPair = 'bruteforce|known|pair|';
    private $prefixFail = 'bruteforce|fail|';
    private $prefixReport = 'bruteforce|report|';

    private $knownIPExpiration = 604800;
    private $knownPairExpiration = 86400; // pair IP:Email
    private $failExpiration = 3600;
    private $reportExpiration = 86400; // 1 days

    public function __construct(Client $redis)
    {
        $this->redis = $redis;
    }

    public function isKnown(string $ip): bool
    {
        return $this->redis->exists($this->prefixKnown.$ip) === 1;
    }

    public function addFail(string $ip): int
    {
        $key = $this->prefixFail.$ip;

        $res = $this->redis->incr($key);
        $this->redis->expire($key, $this->failExpiration);

        return $res;
    }

    public function addKnown(string $ip): void
    {
        $this->redis->setex($this->prefixKnown.$ip, $this->knownIPExpiration, 1);
    }

    public function addKnownPair(string $ip, string $email, string $description): void
    {
        $this->redis->setex($this->prefixKnownPair.$email.':'.$ip, $this->knownPairExpiration, date('Y-m-d H:i:s').' '.$description);
    }

    public function countKnownPairs(string $email): int
    {
        return count($this->redis->keys($this->prefixKnownPair.$email.':*'));
    }

    public function getKnownPairs(string $email)
    {
        $keys = $this->redis->keys($this->prefixKnownPair.$email.':*');
        $values = $this->redis->mget($keys);

        // remove redis prefix from each key
        $keys = array_map(function ($k) { return substr($k, strlen($this->prefixKnownPair)); }, $keys);

        return array_combine($keys, $values);
    }

    public function removeKnownPairs(string $email)
    {
        $keys = $this->redis->keys($this->prefixKnownPair.$email.':*');
        $this->redis->del($keys);
    }

    public function removeFails(string $ip): void
    {
        $this->redis->del([$this->prefixFail.$ip]);
    }

    public function reported(string $ip): void
    {
        $this->redis->setex($this->prefixReport.$ip, $this->reportExpiration, 1);
    }

    public function isReported(string $ip): bool
    {
        return $this->redis->exists($this->prefixReport.$ip);
    }
}
